import {
    BadRequestException,
    HttpException,
    HttpStatus,
    Injectable,
    InternalServerErrorException,
    NotFoundException,
    UnauthorizedException
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { ResponseUtils } from 'src/utils/response.utils';
import JwtHelper from '../../core/jwt/jwt.helper';
import { Constants } from '../../utils/constants';
import CryptoUtils from '../../utils/crypto.utils';
import { CreateUserDto } from '../users/dto/create-user.dto';
import { User } from '../users/entities/user.entity';
import { UsersRepository } from '../users/user.repository';
import { LoginAuthDto } from './dto/login-auth.dto';
import { RegisterAuthDto } from './dto/register-auth.dto';
import { ResetPasswordRequestDto } from './dto/reset-password.dto';
import { v4 as uuidv4 } from 'uuid';
import { ChangePasswordTokenRepository } from '../change-password-token/change-password-token.repository';
import { CreateChangePasswordTokenDto } from '../change-password-token/dto/create-change-password-token.dto';
import { MailSenderService } from '../mail-sender/mail-sender.service';
import { ResetPasswordMailDto } from '../mail-sender/dto/create-reset-password-mail.dto';
import { EmailNotifierRepository } from '../email-notifier/email-notifier-repository';
import { CheckVerifyHashPasswordDto } from './dto/check-verify-hash-password-token';
import { UpdatePasswordUsingVerifyHashPasswordTokenDto } from './dto/update-password-using-verify-hash-password-token';
import { GoogleLoginDto } from './dto/create-google-login.dto';

// import { OrderRepository } from '../orders/order/order-repository';

@Injectable()
export class AuthService {

    constructor(
        @InjectRepository(UsersRepository)
        private readonly repository: UsersRepository,
        @InjectRepository(ChangePasswordTokenRepository)
        private readonly changePasswordTokenRepo: ChangePasswordTokenRepository,
        private readonly jwtHelper: JwtHelper,
        private readonly servMail: MailSenderService,
        private readonly emailNotifierRepository: EmailNotifierRepository,
        // @InjectRepository(OrderRepository)
        // private readonly orderRepo: OrderRepository
    ) {
    }

    async login(dto: LoginAuthDto): Promise<any> {
        const { email, password } = dto;
        const data = await this.repository.findByEmailAddressAndActive(email, 3) || await this.repository.findByEmailAddressAndActive(email, 2);

        if (data instanceof Error) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        if (!data) {
            throw new HttpException(
                "User not created",
                HttpStatus.NOT_FOUND
            );
        }

        const resData = await this.authenticateUser(
            password,
            data
        )
        if (resData == null) {
            // handle unauth
            throw new UnauthorizedException(
                Constants.UNAUTH_REQ
            );
        }
        return resData;
    }

    async loginAdmin(dto: LoginAuthDto): Promise<any> {
        const { email, password } = dto;

        const data = await this.repository.findByEmailAddressAndActive(email, 2);

        if (data instanceof Error) {
            throw new NotFoundException(
                "User not found"
            )
        }
        if (!data) {
            throw new BadRequestException(
                "User not created"
            )
        }

        const resData = await this.authenticateUser(
            password,
            data
        )
        if (resData == null) {
            // handle unauth
            throw new UnauthorizedException(
                Constants.UNAUTH_REQ
            );
        }
        return resData;
    }

    async authenticateUser(
        password: string,
        user: User
    ): Promise<any> {
        const { password: passwordHash, ...entityWithoutPassword } = user;
        const matched = await CryptoUtils.compare(password, passwordHash);
        if (matched) {
            const token = this.jwtHelper.generateToken(
                {
                    userId: entityWithoutPassword.id
                },
                '7d'
            );
            return {
                token: token,
                user: entityWithoutPassword
            };
        }
        return null;
    }

    async register(dto: RegisterAuthDto): Promise<User | Error> {
        const {
            firstName,
            lastName,
            email,
            password,
            phone,
            image,
            company,
            city,
            zipCode,
            country,
        } = dto;

        const isEmailExists = await this.repository.findByEmailAddress(email)
        if (isEmailExists) {
            throw new HttpException(
                "Email Already Exists",
                HttpStatus.BAD_REQUEST
            );
        }
        const createUserDto = {
            firstName,
            lastName,
            email,
            password,
            phone,
            image,
            company,
            city,
            zipCode,
            country,
            userType: "customer"

        } as CreateUserDto;
        const data = await this.repository.createEntity(createUserDto);


        if (data instanceof Error) {
            return new InternalServerErrorException(data)
        } else {
            const loginData = await this.login({
                email: email,
                password: password
            });
            // commenting for now start
            //user getting email
            // let mailDto = {
            //     to: [email],
            //     from: process.env.EMAIL_SENDER_MAIL,
            //     subject: 'Hyman Fashion User Account Creation Notification',
            //     username: firstName + " " + lastName,
            //     password: password,
            //     email: email
            // };

            // const userMail = await this.servMail.accocuntCreateCustomerNotify(mailDto as CreateMailSenderDto);
            // const admins: any = await this.emailNotifierRepository.findOnlyEmail();

            // if (admins && admins.length > 0) {
            //     admins.forEach(async (element: any) => {
            //         let mailAdminDto = {
            //             to: [element.email],
            //             from: process.env.EMAIL_SENDER_MAIL,
            //             subject: 'New Customer Notification.',
            //             name: firstName + " " + lastName,
            //             email: email
            //         };
            //         const adminMail = await this.servMail.adminNotify(
            //             mailAdminDto as AdminNotifyDto,
            //         );
            //     });
            // }
            // commenting for now end
            return ResponseUtils.successResponseHandler(201, "Customer registered successfully!", "data", loginData);
        }

    }

    async changePassword(currentPassword: string, newPassword: string, request: any): Promise<User | Error> {
        const userId = request.user.id;
        const user = await this.repository.findOne(userId);
        const compare = await CryptoUtils.compare(currentPassword, user.password);

        if (!compare) {
            throw new NotFoundException(
                "Password Does not Match",
            )
        }
        await this.repository.update(userId, { password: await CryptoUtils.encrypt(newPassword) });

        return ResponseUtils.successResponseHandler(200, "Password change successfully!", "data", "");
    }

    async forgotPassword(dto: ResetPasswordRequestDto): Promise<User | Error> {
        let userCheck = await this.repository.findByEmailAddress(dto.email);
        if (!userCheck) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        const tokenId = uuidv4();
        let cpr = {
            email: dto.email,
            token: tokenId
        };
        let data = await this.changePasswordTokenRepo.createChangePasswordToken(cpr as CreateChangePasswordTokenDto);

        let mailDto = {
            "to": [
                dto.email
            ],
            "from": process.env.EMAIL_SENDER_MAIL,
            "subject": "Hyman Fashion password reset request",
            "username": userCheck['fullName'],
            "token": cpr.token,
            "email": dto.email
        }

        this.servMail.sendPasswordResetRequestMail(mailDto as ResetPasswordMailDto);

        return ResponseUtils.buildData(
            true,
        );
    }


    async validateRequest(req: any): Promise<boolean> {
        try {
            const bearerToken = req.headers.authorization;
            const bearer = 'Bearer ';
            if (!bearerToken || !bearerToken.startsWith(bearer)) {
                return false;
            }
            const token = bearerToken.replace(bearer, '');
            const jwtPayload = this.jwtHelper.verifyToken(token);
            const data = await this.repository.findOne({
                where: {
                    id: jwtPayload.userId,
                    isDeleted: false
                }
            });
            if (!data || data instanceof Error) {
                return false;
            }

            req.data = data;

            return true;
        } catch (e) {
            return false;
        }
    }

    async verifyHashPasswordTokenRequest(dto: CheckVerifyHashPasswordDto) {
        let userCheck = await this.changePasswordTokenRepo.findByHashToken(dto.hashPass);
        if (!userCheck) {
            return ResponseUtils.buildData(false);
        } else {
            return ResponseUtils.buildData(true);
        }
    }

    async updatePasswordUsingTokenRequest(
        dto: UpdatePasswordUsingVerifyHashPasswordTokenDto
    ) {
        let userCheck = await this.changePasswordTokenRepo.findByHashToken(dto.passToken);
        if (!userCheck) {
            return ResponseUtils.buildData(false);
        } else {
            let userInfo = await this.repository.findByEmailAddress(userCheck['email']);
            if (!userInfo) {
                throw new HttpException(Constants.NOT_FOUND, HttpStatus.NOT_FOUND);
            }
            let updPass = await this.repository.updateUserPassword(
                userInfo['id'],
                await CryptoUtils.encrypt(dto.password)
            );
            return ResponseUtils.buildData(true);
        }
    }

    // async checkNewUser(userId: string): Promise< any | Error>{
    //     try{
    //         const order = await this.orderRepo.find({
    //             where: {
    //                 customerId: userId,
    //                 isDeleted: false
    //             }
    //         });

    //         if(Object.keys(order).length > 0){
    //             return {
    //                 isNewUser: false
    //             }
    //         }
    //         else{
    //             return {
    //                 isNewUser: true
    //             }
    //         }
    //     }
    //     catch(e){
    //         return e;
    //     }
    // }

    async doGoogleLoginService(dto: GoogleLoginDto) {
        let loginResponse = null;
        let checkUserEmail = await this.repository.findByEmailAddress(dto.email);
        if (typeof checkUserEmail === 'undefined') {
            console.log("New User Created");
            let userCreateDto = {
                firstName: dto.name,
                lastName: dto.name,
                email: dto.email,
                password: dto.token,
                phone: "",
                image: "",
                company: "",
                city: "",
                zipCode: "",
                country: "",
                userType: "customer",
                role: 3,
                discountType: "",
                discountAmount: ""
            };

            const data = await this.repository.createEntity(
                userCreateDto as CreateUserDto
            );
            if (!data) {
                console.log("Data User Not Created");
            } else {
                console.log("Data =", data);
            }

            let dtoLogin = {
                email: dto.email,
                password: dto.token
            };
            loginResponse = await this.login(dtoLogin as LoginAuthDto);
        } else {
            console.log("User Already exists");
            await this.repository.updateUserPassword(
                checkUserEmail["id"],
                await CryptoUtils.encrypt(dto.token)
            );
            let dtoLogin = {
                email: dto.email,
                password: dto.token
            };
            loginResponse = await this.login(dtoLogin as LoginAuthDto);
        }
        return loginResponse;
    }

}
